program NkV;

{$IMPORTEDDATA OFF}
{$R *.RES}

uses ActiveX, NTypes, NLib, NSys, NApp, NConf, NForm, NCommand, NResource,
    MainForm, FileList;

type
  TNkVApp = class(TApp)
    procedure Init; override;
    destructor Destroy; override;
  end;

procedure TNkVApp.Init;
var
  i: Integer;
  opt: WideChar;
  nflag: Boolean;
  keymap: TConf;
  list: TFileList;
  stdin: Boolean;
  path: WideString;
begin
  nflag := False;

  with ArgParser do begin
    while GetOpt(opt) do
      case opt of
        'n': nflag := True;
      else
        Error(Format('invalid option -- {}', [opt]));
        ExitCode := 1;
        Exit;
      end;
    Argv := Args;
  end;

  ProfDir := GetEnv('NKTOOLSPROF');
  if ProfDir = '' then
    ProfDir := DirName(ProgPath)
  else
    ProfDir := JoinPath([ProfDir, 'nkv']);

  Config := TConf.Create;
  Config.Read(ResourceText('CONFIG'));
  Config.LoadFromFile(JoinPath([ProfDir, 'position']));
  Config.LoadFromFile(JoinPath([ProfDir, 'config']));

  if Config.GetBool('remote') and (Length(Argv) > 0) then begin
    for i := 0 to High(Argv) do
      Argv[i] := AbsPath(Argv[i]);
    if SendToRemoteForm(TNkVMainForm, JoinCommandLine(Argv)) then
      Exit;
  end;

  keymap := TConf.Create;
  keymap.Read(ResourceText('KEYMAP'));
  keymap.LoadFromFile(JoinPath([ProfDir, 'keymap']));

  MainForm := TNkVMainForm.Create;
  with MainForm as TNkVMainForm do begin
    SetRemoteName;
    Command.BindKeys(keymap);
    keymap.Free;
    Accel := Command.AccelHandle;

    if nflag then begin
      FitImage := False;
      FitLargeImage := False;
      FitWindowOnLoad := True;
    end;

    list := TFileList.Create;
    for i := 0 to High(Argv) do begin
      path := AbsPath(Argv[i]);
      if IsDir(path) then begin
        DirMode := High(Argv) = 1;
        try
          list.AddDir(path);
        except
          on E: EOSError do Error(E.Path, E.Msg);
        end;
      end else
        list.Add(path);
    end;
    list.Sort;
    stdin := list.Count = 0;
    if stdin then
      list.Add('');
    LoadFirstFile(list);
    list.Free;

    if not stdin then
      if FileList.Count = 0 then begin
        ExitCode := 1;
        Exit;
      end else if FileList.Count = 1 then begin
        DirMode := True;
        FileListUpdate := True;
      end;
    Show;
  end;

  Run;
end;

destructor TNkVApp.Destroy;
begin
  MainForm.Free;
  Config.Free;
end;

begin
  CoInitializeEx(nil, COINIT_APARTMENTTHREADED or COINIT_DISABLE_OLE1DDE);
  App := TNkVApp.Create('NkV');
  try
    try
      App.Init;
    except
      App.Fatal;
    end;
  finally
    App.Free;
    CoUninitialize;
  end;
end.
